==> libetherx-0.9pre4.tar.gz: libetherx-0.9pre4/src/ehandler.c <==
/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  Jabber
 *  Copyright (C) 1998-1999 The Jabber Team http://jabber.org/
 */

#include <etherxd.h>

char *etherx__host = NULL;
char *etherx__secret = NULL;

void etherx_init(char *secret, char *host)
{
    etherx__host = strdup(host);
    etherx__secret = strdup(secret);
    log_debug(ZONE,"etherx_init(%s,%s)",secret,host);
}

void ehandler_connect(ehandler eh)
{ /* we block until success or failure */
    static int daemon = 0;

    log_debug(ZONE,"ehandler_connect!");

    /* if we started a working daemon already */
    if(daemon)
    {
        etherxd_new(eh); /* tell it about a new transport handler */
        return;
    }

    /* attempt an xmlstream, unlocks when connected or error */
    etherxs_connect(eh);

    log_debug(ZONE,"spun off connection attempt");

    /* wait for lock from stream connect, and then we don't need it anymore */
    pth_mutex_acquire(&(eh->mutex), FALSE, NULL);
    pth_cond_await(&(eh->cond), &(eh->mutex), NULL);
    log_debug(ZONE,"condition happened! eh->type is %d",eh->type);
    pth_mutex_release(&(eh->mutex));

    /* we're connected as a stream */
    if(eh->type != EHANDLER_ERROR)
        return;

    log_debug(ZONE,"no luck connecting");

    /* attempt a listen daemon */
    etherxd_new(eh);

    /* if it's running, flag for future use */
    if(eh->type == EHANDLER_DAEMON)
        daemon = 1;

    log_debug(ZONE,"returning eh->type %d",eh->type);
}

ehandler ehandler_new(etherx_handler h, char *host, char *namespace)
{
    pool p;
    ehandler e;

    log_debug(ZONE,"ehandler_new(%s,%s)",host,namespace);

    /* init new ehandler */
    p = pool_new();
    e = pmalloc(p, sizeof(_ehandler));
    e->host = pstrdup(p, host);
    e->namespace = pstrdup(p, namespace);
    e->p = p;
    e->h = h;
    e->t = NULL;
    e->type = EHANDLER_ERROR;

    /* we're not a true handler, just a stub */
    if(h == NULL)
        return e;

    /* init the mutex and try connecting */
    if(pth_mutex_init(&(e->mutex)) && pth_cond_init(&(e->cond)))
        ehandler_connect(e);

    if(e->type != EHANDLER_ERROR)
        return e;

    log_error("libetherx","Failed to connect or become daemon");
    pool_free(p);
    return NULL;
}

void ehandler_send(ehandler eh, xmlnode x, char *to)
{
    epacket ep;

    if(eh == NULL || x == NULL || to == NULL)
    {
        log_warn("libetherx","received illegal outgoing data");
        return;
    }

    ep = epacket_new(x, to, eh->host, eh->namespace);

    if(eh->type == EHANDLER_SOCKET)
    {
        log_debug(ZONE,"ehandler_send: socket %s",xmlnode2str(x));
        xmlstream_write(eh->t, epacket_serialize(ep));
        return;
    }
    if(eh->type == EHANDLER_DAEMON)
    {
        log_debug(ZONE,"ehandler_send: daemon %s",xmlnode2str(x));
        etherxd_send(ep);
        return;
    }

    /* if we get here it's because the type is EHANDLER_ERROR from the socket disconnecting, bounce back to the handler */
    epacket_error(ep,"delivery failure");
    (*eh->h)(ep);
}

==> libetherx-1.0.tar.gz: libetherx-1.0/src/ehandler.c <==
/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  Jabber
 *  Copyright (C) 1998-1999 The Jabber Team http://jabber.org/
 */

#include <etherxd.h>

char *etherx__remote = NULL;
char *etherx__secret = NULL;
thread etherx__daemon = NULL;
ehandler etherx__handlers = NULL;

void etherx_init(char *secret, char *host)
{
    if(host != NULL)
        etherx__remote = strdup(host);
    etherx__secret = strdup(secret);
}

ehandler ehandler_new(etherx_handler h, char *host, char *namespace)
{
    pool p;
    ehandler e;

    log_debug(ZONE,"ehandler_new(%s,%s)",host,namespace);

    /* init new ehandler */
    p = pool_heap(1*1024);
    e = pmalloc(p, sizeof(_ehandler));
    e->host = pstrdup(p, host);
    e->namespace = pstrdup(p, namespace);
    e->p = p;
    e->h = h;
    e->t = NULL;
    e->attempts = 0;

    /* we're not a true handler, just a stub */
    if(h == NULL)
        return e;

    /* keep a master list of all the handlers that want to receive data */
    if(host != NULL)
        etherx__handlers = ehandler_list_add(etherx__handlers,e);

    /* this process is the daemon, we don't need to do anything else */
    if(etherx__daemon != NULL)
        return e;

    /* try connecting a new xmlstream */
    etherxs_connect(e);

    log_debug(ZONE,"spun off connection attempt");

    return e;
}

void ehandler_send(ehandler eh, xmlnode x, char *to)
{
    epacket ep;

    if(eh == NULL || x == NULL || to == NULL)
    {
        log_warn("etherx","received illegal outgoing data");
        return;
    }

    ep = epacket_new(x, to, eh->host, eh->namespace);

    /* running daemon */
    if(etherx__daemon)
    {
        log_debug(ZONE,"ehandler_send: daemon %s",xmlnode2str(x));
        etherxd_send(ep);
        return;
    }

    /* connected socket */
    if(eh->t != NULL)
    {
        log_debug(ZONE,"ehandler_send: socket %s",xmlnode2str(x));
        xmlstream_write(eh->t, epacket_serialize(ep));
        return;
    }

    /* if we get here it's because the socket isn't connected, bounce back to the handler */
    epacket_error(ep,"delivery failure");
    (*eh->h)(ep);
}
